"""
Ref: The Schrodinger Equation: First Steps
     By Graham W Griffiths, May 2023, available at: www.pdecomp.net 
This python code produces 3D Wave Function and Probability Density Function
plots for the 3D Spherical Potential Well - Figs. 4.8 and 4.9.
Refs: https://www.hasenkopf2000.net/wiki/page/3d-hydrogen-structure-python/
      D Griffiths (1995), Introduction to Quantum Mechanics, Prevntice Hall.
"""

import numpy as np
import scipy.special
import scipy.misc
import psiSlice as ps
# import vtk
# import PyQt5
from mayavi import mlab
# from mlabtex import mlabtex # bugs!

# Bessel function zeros, Beta(n,l)
Beta = np.array( [ [  0.0,         0.0,         0.0,         0.0,         0.0       ],   # n=0, invalid!
                   [  3.14159265,  4.49340946,	5.7634592 ,	 6.987932  ,  8.18256145],   # n=1
                   [  6.28318531,  7.72525184,	9.09501133,	10.41711855, 11.70490715],   # n=2
                   [  9.42477796, 10.90412166, 12.32294097,	13.69802315, 15.03966471],   # n=3
                   [ 12.56637061, 14.06619391, 15.51460301,	16.92362129, 18.30125596] ]) # n=4

# Radial normalization constants for radius a=1e-8
AR = np.array([ [2.50662827e+08, 3.07114233e+08, 3.55836766e+08, 3.99798315e+08],
                [3.54490770e+08, 3.96350791e+08, 4.34438149e+08, 4.69781560e+08],
                [4.34160753e+08, 4.68952665e+08, 5.01421955e+08, 5.32052952e+08],
                [5.01325655e+08, 5.31738401e+08, 5.60542224e+08, 5.88006826e+08]])


AY = np.array([ [ 1.0, 1.0, 1.0, 1.0],
                [ 0.0, 1.0, 1.0, 1.0],
                [ 0.0, 0.0, 1.0, 1.0],
                [ 0.0, 0.0, 0.0, 1.0 ] ])



def r(x,y,z):
  r1 = np.sqrt(x**2+y**2+z**2)
  r = np.select([r1<0.0, r1>a], # mask values greater than a
                [0.0, a],default=r1)
  return(r)
theta = lambda x,y,z: np.arccos(z/r(x,y,z)) # [0-pi]
phi   = lambda x,y,z: np.pi+np.select(        # [0-2pi]
                             	       [x>0, x==0, x<0],
                             	       [
                                  		np.arctan(y/x),
                                  		0.5*np.pi*np.sign(y),
                                  		np.arctan(y/x)+np.pi*np.sign(y)]
                                      )

R = lambda r, l: scipy.special.spherical_jn(l, Bnl*r/a)
WF = lambda r,theta,phi,n,l,m: ARnl*R(r,l) * \
                               AYml*scipy.special.sph_harm(m,l,phi,theta)
realWF = lambda r,theta,phi,n,l,m: WF(r,theta,phi,n,l,m).real
absWF_sqr = lambda r,theta,phi,n,l,m: abs(WF(r,theta,phi,n,l,m))**2

a = 1e-8 # Radius of "spherical potential well"
Xmin =-a; Xmax = a
Ymin =-a; Ymax = a
Zmin =-a; Zmax = a

N = 201j
x,y,z = np.mgrid[Xmin:Xmax:N,Ymin:Ymax:N,Zmin:Zmax:N]

# Set up mask to show contours
mask1 = np.select([phi(x,y,z)>np.pi*3/2], [np.nan],default=np.pi*3/2)
mask2 = np.select([phi(x,y,z)>np.pi*6/4], [np.nan],default=np.pi*6/4)
mask3 = np.select([phi(x,y,z)>np.pi], [np.nan],default=np.pi)
mask = mask2
# mask=1 # tmp = w[~np.isnan(w)] # chk no negative values
# Set quantum numbers
n, l, m = [1,3,2] # Note: n starts at 1 for consistency with the literature.
if n <1 or n>4:
  raise Exception("n must be >= 1 and <= 4")
if m > l:
  raise Exception("m must be <= l")
Bnl  = Beta[n,l]  # nth zero of order-l spherical Bessel function: 
AYml = AY[m,l]    # normalization constant: angular
ARnl = AR[n,l]    # normalization constant: radial

# Set solution type: with or without contours
solType = 0
if solType == 0:
  w = absWF_sqr(r(x,y,z),theta(x,y,z),phi(x,y,z),n,l,m); txtFunc = "pdf"
  col = 'rainbow'
else:
  w = realWF(r(x,y,z),theta(x,y,z),phi(x,y,z),n,l,m); txtFunc = "psi"
  col = 'jet'
  
# Set up 3d Contour plot
mlab.figure(size=[800,800], bgcolor=(1,1,1), fgcolor=(0.,0.,0.)) # size in px
p = mlab.contour3d(x,y,z, w*mask,contours=20,transparent=False, \
                    reset_zoom=True,colormap=col) #True)
#mlab.gcf().scene.parallel_projection = True
#mlab.view(azimuth=142, elevation=48, distance=1e-7)
mlab.view(azimuth=124, elevation=60, distance=1e-7)
p.scene.parallel_projection = True
p.scene.camera.parallel_scale = 2.5e-8
n_text = f"{txtFunc}: n={n}, l={l}, m={m}"; print(n_text) # Title

mlab.title(n_text,size=0.6, height=0.9)


mlab.axes() # Show axes
cb = mlab.colorbar(orientation='vertical', nb_labels=5, label_fmt='%.2e')
cb.scalar_bar.unconstrained_font_size = True
cb.label_text_property.font_size=18
# mlab.colorbar(orientation='horizontal', nb_labels=5)
mlab.outline() 
#mlab.show()

# Plot through cross-section of Solution
title = f"$|\psi|^2$:   n={n}, l={l}, m={m}"
Nmid = np.ceil((np.shape(x)[0])/2)
# ps.psi_slice(x,z,w,a,Nmid,title)








