"""
Ref: The Schrodinger Equation: First Steps
     By Graham W Griffiths, May 2023, available at: www.pdecomp.net 
This python code produces 'even parity' grahical solution plots for 1D Finite
Potential Well - Fig. 5.2.

"""

import numpy as np
from matplotlib import pyplot as plt
from scipy.optimize import root # to solve for intersections

# Data
L = 1e-9         # [m]
m = 9.11e-31     # [kg]
hbar = 1.055e-34 # [J s]
z0 = 10
V0 = (1/(2*m))*(z0*hbar/L)**2
# V0 = 3.81*1.6021E-19 # [J] converted from [eV], 3.81 [eV]
# z0 = np.sqrt(2*m*V0)*L/hbar

z = np.linspace(0,z0+2,500)

t = np.tan(z)
tmp = np.abs((z0/(z+1e-6))**2 -1)
t2 = np.sqrt(tmp)

# Insert a NaN where the difference between successive points is negative
# to aV0id drawing vertical lines for t
t[:-1][np.diff(t) < 0] = np.nan

fig = plt.figure(figsize=(8,4))
ax = fig.add_subplot(111)
ax.plot(z, t,  'g', lw=2, label=r'$\tan(z)$')
ax.plot(z, t2, 'b', lw=2, label=r'$\sqrt{(z_0/z)^2-1}$')
ax.set_xlabel(r'$z$', fontsize=12)
ax.set_ylabel(r'$\tan(z)$,  $\sqrt{(z_0/z)^2-1}$', fontsize=12)

ax.set_xlim([0,z0+0.4])
ax.set_ylim([0,5*np.pi/2])
zl = [0,np.pi/2,np.pi,3*np.pi/2,2*np.pi,5*np.pi/2,z0]
zlabels = [r'$0$',r'$\pi/2$',r'$\pi$',r'$3\pi/2$',r'$2\pi$', \
           r'$5\pi/2$      ',r'  $z_0$']
ax.set_xticks(zl, zlabels)
for v in [1,3,5]:
  ax.axvline(x = zl[v], color='g', linestyle='--',lw=2)
ax.grid()
ax.set_title(f'$z_0=${z0}', fontsize=15)
fig.tight_layout()

# Solve for intersection points, [zz, zt], and add to plot
def func1(z, z0):
    return np.tan(z)-np.sqrt(((z0/(z))**2 -1))

zz = []; zt = []
for x0 in [0.5,4.0,6.5,9.0]:
  sol = root(func1, x0, args=(z0, ),tol=1e-14).x[0] # solve for roots
  zz += [round(sol,14)]
  zt += [round(np.tan(sol),14)]
# print(f'zz = {zz}') # [1.428, 4.271, 7.069, 9.679]
# print(f'zt = {zt}') # [6.933, 2.117, 1.001, 0.260]

ax.plot(zz,zt,'ko', label=f'solutions, $z_n$')
ax.legend()#loc=[.63,.7],facecolor='w')

# Calculate energy levels

print(f'V0 = {V0}')
En = (1/(2*m)*(hbar*np.array(zz)/L)**2) - V0 # [J]
print(f'En = {En}')
print(f'En = {En/1.6021E-19}')




