"""
Ref: The Schrodinger Equation: First Steps
     By Graham W Griffiths, May 2023, available at: www.pdecomp.net 
This python code produces 'odd parity' grahical solution plots for 1D Finite
Potential Well - Fig. 5.3.

"""

import numpy as np
from matplotlib import pyplot as plt
from scipy.optimize import root # to solve for intersections


# Data
L = 1e-9         # [m]
m = 9.11e-31     # [kg]
hbar = 1.055e-34 # [J s]
z0 = 10
V0 = (1/(2*m))*(z0*hbar/L)**2
# V0 = 3.18*1.6021E-19 # [J] converted from [eV], 3.81 [eV]
# z0 = np.sqrt(2*m*V0)*L/hbar

z = np.linspace(0,z0,500)

t = -1/(np.tan(z)+1e-12)
t2 = np.sqrt((z0/(z+1e-12))**2 -1)

# Insert a NaN where the difference between successive points is negative
# to avoid drawing vertical lines for t
t[:-1][np.diff(t) < 0] = np.nan

fig = plt.figure(figsize=(8,4))
ax = fig.add_subplot(111)
ax.plot(z, t,  'g', lw=2, label=r'$-\cot(z)$')
ax.plot(z, t2, 'b', lw=2, label=r'$\sqrt{(z_0/z)^2-1}$')
ax.set_xlabel(r'$z$', fontsize=12)
ax.set_ylabel(r'-$\cot(z)$,  $\sqrt{(z_0/z)^2-1}$', fontsize=12)

ax.set_ylim([0,5*np.pi/2])
zl = [0,np.pi,2*np.pi,3*np.pi,z0]
zlabels = [r'$0$',r'$\pi$',r'$2\pi$',r'$3\pi$',r'$z_0$']
ax.set_xticks(zl, zlabels)
for v in [1,2,3]:
  ax.axvline(x = zl[v], color='g', linestyle='--',lw=2)
ax.grid()
ax.set_title(f'$z_0=${z0}', fontsize=15)
fig.tight_layout()

# Solve for intersection points, [zz, zt], and add to plot
def func2(z, z0):
    return -1/np.tan(z)-np.sqrt(((z0/(z))**2 -1))

zz = []; zt = []
for x0 in [2.0,4.5,7.0]:
 sol = root(func2, x0, args=(z0, )).x[0] # solve for roots
 zz += [round(sol,14)]
 zt += [round(-1/np.tan(sol),14)]
# print(f'zz = {zz}') # [2.852, 5.679, 8.423]
# print(f'zt = {zt}') # [3.36 , 1.449, 0.64 ]
ax.plot(zz,zt,'ko', label=f'solutions = $z_n$')
ax.legend(loc=[.66,.7],facecolor='w')

# Calculate energy levels

#V0 = (1/(2*m))*(z0*hbar/L)**2
print(f'V0 = {V0}')
En = (1/(2*m)*(hbar*np.array(zz)/L)**2) - V0 # [J]
print(f'En = {En}')
print(f'En = {En/1.6021E-19}')








