"""
Ref: The Schrodinger Equation: First Steps
     By Graham W Griffiths, May 2023, available at: www.pdecomp.net 
This python code produces Wave Function plots for 1D Finite Potential Well
 - Fig. 5.4.
Code adapted from:
Ref: https://chem.libretexts.org/Ancillary_Materials/Interactive_Applications/Jupyter_Notebooks/Particle_in_a_Finite_Potential_Box_(Python_Notebook)
NOTE: Changed to be a negative potential, so energy levels = Evals - Vo, 
      to conform to D Griffiths (1995), Introduction to Quantum Mechanics,
      Prevntice Hall.

"""
import numpy as np
from matplotlib import pyplot as plt

def WF_plots(L,E_vals,V1):
  
  Vo = V1/1.6021E-19
  # val is equal to sqrt(2m*1eV)*1A/(2*hbar)
  val = np.sqrt(2.0*9.10938356e-31*1.60217662e-19)* \
                1e-10/(2.0*1.05457180013e-34)
  n = np.size(E_vals)
  nstates = n
  cols = ["blue","black"] #["red", "blue"]
  fig, ax = plt.subplots(figsize=(12,9))
  ax.spines['right'].set_color('none')
  ax.xaxis.tick_bottom()
  ax.spines['left'].set_color('none')
  ax.axes.get_yaxis().set_visible(False)
  ax.spines['top'].set_color('none')
  X_lef = np.linspace(-L, -L/2.0, 900,endpoint=True)
  X_mid = np.linspace(-L/2.0, L/2.0, 900,endpoint=True)
  X_rig = np.linspace(L/2.0, L, 900,endpoint=True)
  ax.axis([-L,L,-0.1,1.1*Vo])
  ax.set_xlabel(r'$X$ (Angstroms)',fontsize=20)
  ax.tick_params(axis='x', which='major', labelsize=20)
  str1="$V_o = %.3f$ eV"%(-Vo)
  #ax.text(2.0*L/2.0, 1.02*Vo, str1, fontsize=24, color=cols[1])
  ax.text(L+1, -0.2, str1, fontsize=24, color=cols[1])
  # Defining the maximum amplitude of the wavefunction
  if (nstates > 1):
      amp = np.sqrt((E_vals[1]-E_vals[0])/1.5)/2
  else:
      amp = np.sqrt((Vo-E_vals[0])/1.5)/2
  print(E_vals - Vo)
  # Plotting the wavefunctions
  for n in range(1,nstates+1):
    ax.hlines(E_vals[n-1], -L, L, linewidth=1.8, linestyle='--', color="black")
    str1="$n = "+str(n)+r"$, $E_"+str(n)+r" = %.3f$ eV"%(E_vals[n-1]-Vo)
    # ax.text(2.0*L/2.0, E_vals[n-1]+0.01*Vo, str1, fontsize=20, color=cols[1])
    ax.text(L+1, E_vals[n-1]+0.01, str1, fontsize=20, color=cols[1])
    k = 2.0*np.sqrt(E_vals[n-1])*val
    a0 = 2.0*np.sqrt(Vo-E_vals[n-1])*val
    # Plotting odd wavefunction
    if (n%2==0):
      ax.plot(X_lef,E_vals[n-1]-amp*np.exp(a0*L/2.0)*np.sin(k*L/2.0)*np.exp(a0*X_lef), color=cols[0], label="", linewidth=2.8)
      ax.plot(X_mid,E_vals[n-1]+amp*np.sin(k*X_mid), color=cols[0], label="", linewidth=2.8)
      ax.plot(X_rig,E_vals[n-1]+amp*np.exp(a0*L/2.0)*np.sin(k*L/2.0)*np.exp(-a0*X_rig), color=cols[0], label="", linewidth=2.8)
    # Plotting even wavefunction
    else:
      ax.plot(X_lef,E_vals[n-1]+amp*np.exp(a0*L/2.0)*np.cos(k*L/2.0)*np.exp(a0*X_lef), color=cols[0], label="", linewidth=2.8)
      ax.plot(X_mid,E_vals[n-1]+amp*np.cos(k*X_mid), color=cols[0], label="", linewidth=2.8)
      ax.plot(X_rig,E_vals[n-1]+amp*np.exp(a0*L/2.0)*np.cos(k*L/2.0)*np.exp(-a0*X_rig), color=cols[0], label="", linewidth=2.8)
  ax.margins(0.00)
  ax.vlines(-L/2.0, -0.1, Vo, linewidth=4.8, color=cols[1])
  ax.vlines(L/2.0, -0.1, Vo, linewidth=4.8, color=cols[1])
  ax.hlines(-0.1, -L/2.0, L/2.0, linewidth=4.8, color=cols[1])
  ax.hlines(Vo, -L, -L/2.0, linewidth=4.8, color=cols[1])
  ax.hlines(Vo, L/2.0, L, linewidth=4.8, color=cols[1])
  plt.title(r'Wavefunctions, $\psi_n(x)$', fontsize=30)
  plt.legend(bbox_to_anchor=(0.8, 1), loc=2, borderaxespad=0.)
  plt.show()
  fig.tight_layout()

##############################################
# Data
L = 1e-9         # [m]
m = 9.11e-31     # [kg]
hbar = 1.055e-34 # [J s]
z0 = 10
V0 = (1/(2*m))*(z0*hbar/L)**2
# V0 = 3.18*1.6021E-19 # [J] converted from [eV], 3.81 [eV]
# z0 = np.sqrt(2*m*V0)*L/hbar

# En1 = np.array([-5.98431733e-19,-5.61180520e-19,-4.99442446e-19,-4.13851033e-19,\
#                 -3.05628459e-19,-1.77458675e-19,-3.86028112e-20])
# En2 = -En1[::-1]/1.6021E-19
# Values for +ve potential - will be changed to -ve in WF_plots()
En2 = np.array([0.0776387,0.309974,0.695033,1.22885,1.90382,2.7032,3.56924])

L2 = (20*L)*1e9

WF_plots(L2,En2,V0)

