"""
Ref: The Schrodinger Equation: First Steps
     By Graham W Griffiths, May 2023, available at: www.pdecomp.net 
This python code produces Quantum Tunneling Transmission and
Reflection Coefficient plots - Fig. 7.2.
See also:
https://www.youtube.com/watch?v=kUR98x1tH0c&ab_channel=ProfessorDaveExplains
"""

import numpy as np
from matplotlib import pyplot as plt


L = 4.5e-10      # [m]
m = 9.11e-31     # [kg]
hbar = 1.055e-34 # [J s]
V0 = 1.6e-18     # [J]
N = 200

eta1 = np.linspace(0.01,0.999, N) # E<V0
eta2 = np.linspace(1.001,4, N)    # E>V0
E1 = eta1*V0
E2 = eta2*V0
T1 = np.zeros(N) # E<V0
T2 = np.zeros(N) # E>V0


for i in range(N):
  k1 = np.sqrt(2*m*(V0-E1[i]))/hbar  
  k2 = np.sqrt(2*m*(E2[i]-V0))/hbar  
  T1[i] = 1/(V0**2*np.sinh(k1*L)**2/(4*E1[i]*(V0-E1[i]))+1)  
  T2[i] = 1/(V0**2*np.sin(k2*L)**2/(4*E2[i]*(E2[i]-V0))+1)
  #print(f"k1 = {k1}, T = {T2[i]}")
  
fig, ax = plt.subplots(figsize=(12,9))
ax.plot(eta1, T1, '--b',linewidth=3,label="T (E<Vo)")
ax.plot(eta2, T2, '-b',linewidth=3,label="T (E>Vo)")
ax.plot(eta1, 1-T1, '--r',linewidth=3,label="R (E<Vo")
ax.plot(eta2, 1-T2, '-r',linewidth=3,label="R (E>Vo")
ax.set_xlabel(r'$\eta=E/V_0$)',fontsize=20)
ax.set_ylabel(r'$T, R$',fontsize=20)
ax.tick_params(axis='both', which='major', labelsize=18)
ax.set_title(r'Transmission and Reflection Coefficients', fontsize=28)
ax.grid()
ax.legend(fontsize=18)
plt.show()
fig.tight_layout()

