"""
Ref: The Schrodinger Equation: First Steps
     By Graham W Griffiths, May 2023, available at: www.pdecomp.net 
This python code produces a plot of the real part of the tunneling problem
solution - Fig. 7.3.
"""



import numpy as np
import sympy as sp
from sympy import symbols, diff, lambdify
from matplotlib import pyplot as plt
import matplotlib.patches as mpatches


L = 1.8e-10 #4.5e-10      # [m]
m = 9.11e-31     # [kg]
hbar = 1.055e-34 # [J s]
V0 = 1.6*10**(-18)     # [J]
BC = 0.6 # barrier coefficient
En = BC*V0
w = En/hbar 
k1 = np.sqrt(2*m*En)/hbar 
kappa2 = np.sqrt(2*m*(V0-En))/hbar

B, F, G, C, x = symbols('B F G C x')
psi1 = sp.exp(1J*k1*x) + B*sp.exp(-1J*k1*x)
psi2 = F*sp.exp(kappa2*x) + G*sp.exp(-kappa2*x)
psi3 = C*sp.exp(1J*k1*x)

exp1 = psi1.subs(x,0) - psi2.subs(x,0)
exp2 = psi2.subs(x,L) - psi3.subs(x,L)
exp3 = diff(psi1,x).subs(x,0) - diff(psi2,x).subs(x,0)
exp4 = diff(psi2,x).subs(x,L) - diff(psi3,x).subs(x,L)

sol = sp.solve((exp1,exp2,exp3,exp4), (B, C, F, G))

# Solution values
# B =  0.1815481747 - 0.9352973513*1J
# C = -0.1436753713 - 0.2676126167*1J
# F =  0.01802377044 + 0.03354866193*1J
# G =  1.163524404  - 0.9688460132*1J

N = 150
x1 = np.linspace(-4*L,0,4*N)
x2 = np.linspace(0,L,N)
x3 = np.linspace(L,L+5*L,4*N)

psi11 = [psi1.subs(B,sol[B]).subs(x,x1[i]) \
          for i in range(len(x1))]
psi22 = [psi2.subs(F,sol[F]).subs(G,sol[G]).subs(x,x2[i]) \
          for i in range(len(x2))]
psi33 = [psi3.subs(C,sol[C]).subs(x,x3[i]) \
          for i in range(len(x3))]

psi111 = np.array(psi11, dtype=complex)
psi222 = np.array(psi22, dtype=complex)
psi333 = np.array(psi33, dtype=complex)

# abs_psi1 = np.sqrt(psi111*np.conj(psi111))
# abs_psi2 = np.sqrt(psi222*np.conj(psi222))
# abs_psi3 = np.sqrt(psi333*np.conj(psi333))

Re_psi1 = np.real(psi111)
Re_psi2 = np.real(psi222)
Re_psi3 = np.real(psi333)



# specify the location of (left,bottom),width,height
rect=mpatches.Rectangle((0,-2.5),L,5, 
                        fill = True, facecolor = "gainsboro",
                        linewidth = 4, edgecolor = "blue",
                        label = "Barrier")
fig, ax = plt.subplots(figsize=(12,6))
ax.add_patch(rect)
ax.plot(x1, Re_psi1, '-b',linewidth=3,label=r"$\psi_1(x)$")
ax.plot(x2, Re_psi2, '-r',linewidth=3,label=r"$\psi_2(x)$")
ax.plot(x3, Re_psi3, '-k',linewidth=3,label=r"$\psi_3(x)$")
ax.set_xlabel(r'$x$',fontsize=20)
ax.set_ylabel(r'$\Re \, \psi(x)$',fontsize=20)
ax.set_ylim([-2,2.5])
ax.tick_params(axis='both', which='major', labelsize=18)
ax.set_title(f'Quantum Tunneling, $V_0=1.6$x$10^{{-18}}$ [J], $E={BC}\,V_0$', fontsize=28)

ax.grid()
ax.legend(fontsize=18)
plt.show()
fig.tight_layout()



