"""
Ref: The Schrodinger Equation: First Steps
     By Graham W Griffiths, May 2023, available at: www.pdecomp.net 
This python code produces Hydrogen Orbital plots for Wave Functions and Probability Density 
functions, Figs 8.3 and 8.4
Adapted from:
  http://staff.ustc.edu.cn/~zqj/posts/Hydrogen-Wavefunction/
"""
import numpy as np
from sympy import lambdify, integrate, oo
from sympy.abc import r, theta, phi
from sympy.physics.hydrogen import R_nl, Psi_nlm
from mayavi import mlab

def cart2sphe(x, y, z):
    '''
    3D Cartesian coordinates to spherical coordinates.

    input:
        x, y, z : numpy arrays
    '''
    xy2     = x**2 + y**2
    r       = np.sqrt(xy2 + z**2)    
    theta   = np.arctan2(np.sqrt(xy2), z) # the polar angle in radian angles
    phi     = np.arctan2(y, x)            # the azimuth angle in radian angles
    phi[phi < 0] += np.pi * 2             # np.arctan2 returns the angle in the
                                          # range [-pi, pi]

    return r, theta, phi

n,l,m = 3,1,0 # set as required
psi  = lambdify((r, phi, theta), Psi_nlm(n, l, m, r, phi, theta), 
                'numpy')

if n == 1:
    L, N = 1.5*n**2 + 1.0, 100
else:
    L, N = 1.5*n**2 + 5.0, 100

a0 = 0.529E-10  
L = 26 # 15, pdf and 26 psi
Lmin = -L*a0
Lmax = L*a0

x,y,z = np.mgrid[
    -L:L:N*1j,
    -L:L:N*1j,
    -L:L:N*1j
]
nx, ny, nz = x.shape

r0, theta, phi = cart2sphe(x, y, z)

wfc = psi(r0, phi, theta).reshape(nx, ny, nz)
abs_wfc  = np.abs(wfc)**2
real_wfc = np.real(wfc) #*a0**(-3/2))

# Set solution type: WF or PDF
solType = 0
if solType == 0:
  w = abs_wfc; txtFunc = "pdf:"
  col = 'rainbow'
  dist = 90
else:
  w = real_wfc; txtFunc = "psi:"
  col = 'jet'
  dist = 230

mask1 = np.select([phi>np.pi*3/2], [np.nan],default=np.pi*3/2)# (0,1,3)(3,3,3)
mask2 = np.select([phi>np.pi*6/4], [np.nan],default=np.pi*6/4)# (0,2,3)
mask = mask2 # select mask to show contours
#mask = 1

mlab.figure(size=[800,800], bgcolor=(1,1,1), fgcolor=(0.,0.,0.)) # size in px
mlab.gcf().scene.parallel_projection = True
p = mlab.contour3d(x*a0,y*a0,z*a0,w*mask,contours=12,transparent=False,
                   reset_zoom=True,colormap=col) #True)
mlab.outline(p ,extent=(Lmin,Lmax ,Lmin,Lmax, Lmin,Lmax) )
#mlab.view(azimuth=130, elevation=70, distance=230) #400)
mlab.view(azimuth=-67, elevation=67, distance=dist*a0) #90,pdf, 130 WF)

n_text = f"{txtFunc} n={n}, l={l}, m={m}"; print(n_text)
mlab.title(n_text,size=0.6, height=0.9)


mlab.axes(extent=(Lmin,Lmax ,Lmin,Lmax, Lmin,Lmax))
cb=mlab.colorbar(orientation='vertical', nb_labels=5)
cb.label_text_property.font_size=16
cb.scalar_bar.unconstrained_font_size = True
#mlab.outline()
# mlab.show()







