"""
Ref: The Schrodinger Equation: First Steps
     By Graham W Griffiths, May 2023, available at: www.pdecomp.net
This python code produces Spherical Bessel Function plots and 
Reduced Spherical Bessel Function plots - Figs. 4.6 and 4.7.
"""

import numpy as np
import matplotlib.pyplot as plt
from scipy.special import jn, spherical_jn


# %matplotlib qt         :in console for plots in separate window
# %matplotlib inline     :in console for inline plots

N = 300
r = np.linspace(0,14,N)

fig, ax = plt.subplots(figsize=(10,6))
ax.axhline(y=0.0, color='k', linestyle='-',linewidth=2)
for i in range(4) :
  ax.plot(r,spherical_jn(i,r),linewidth=3,
  #ax.plot(r, np.sqrt(np.pi/(2*r))*jn(i+0.5,r),linewidth=3,
          label=f"$j_{i}$") # plot the ith Bessel function
ax.set_xlim(0, 14)
ax.set_ylim(-0.4, 1)

ax.xaxis.set_tick_params(labelsize=14)
ax.yaxis.set_tick_params(labelsize=14)
ax.set_xlabel('r', fontsize=18)
ax.set_ylabel('$J_\ell(r)$', fontsize=18)
#plt.legend(['$J_0(x)$','$J_1(x)$','$J_2(x)$','$J_3(x)$','$J_4(x)$','$J_5(x)$'])
leg = ax.legend(facecolor='white', edgecolor='black', fontsize=15, 
          borderpad=1)
leg.get_frame().set_linewidth(2)
ax.grid()
ax.set_title("Spherical Bessel Functions: $J_\ell(x)$", fontsize=20)



# Beta[n,ell] - Spherical Bessel function Zeros
Beta = np.array([ [ 3.14159265,  6.28318531,  9.42477796, 12.56637061],
                  [ 4.49340946,  7.72525184, 10.90412166, 14.06619391],
                  [ 5.7634592 ,  9.09501133, 12.32294097, 15.51460301],
                  [ 6.987932  , 10.41711855, 13.69802315, 16.92362129],
                  [ 8.18256145, 11.70490715, 15.03966471, 18.30125596] ])

Nn = 4
L = 1e-8 #1e-8

R = np.linspace(0, L, N)
fig2, _ = plt.subplots(2,2,figsize=(8, 8))
for ell, axs in enumerate(fig2.axes):
  axs.set_title(f"$\ell = {ell}$",fontsize=20)
  axs.set_xlabel('r',fontsize=16)
  axs.set_ylabel(r'$j_{\ell}(\beta_{n \ell} \, r/a)$',fontsize=16)
  axs.tick_params(axis='both', which='major', labelsize=12)
  for n in np.arange(1, 5):
    Bn = Beta[ell,n-1]; #print(Bn)
    bf = spherical_jn(ell, Bn*R/L)
    axs.plot(R, bf, label=f'$n = {n}$')
    #axs.legend(loc='best')
    leg = axs.legend(loc='upper right',facecolor='white', edgecolor='black', #fontsize=15, 
              borderpad=1)
    leg.get_frame().set_linewidth(2)
plt.tight_layout()
plt.show()

anl = np.sqrt(Beta/L)
print(anl)

